"""
The vpg.py module contains properly-formatted API calls for various Zerto VPG tasks, including 
gathering VPG info such as checkpoints, as well as performing actions such as fail over test, 
or fail over live. 
"""

import requests

class vpgs():
    """
    the vpgs class houses VPG specific methods. 
    ...
    Attributes
    ----------
    zvmip: str
        the IP address of the target ZVM 
    
    headerwithkey : dict
        a properly formatted dict containing the following key:value pairs:
        {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
            'x-zerto-session': str-type containing valid session key generated with zerto_auth.py
        }

    vpgid: str
        unique identifier for individual VPG

    Methods
    -------
    getInfoAllVpgs()
        Returns information for all VPGs
    
    getInfoSingleVPG(vpgid)
        Returns information for individual VPG
    
    getCheckpointsForVPG(vpgid)
        Returns all checkpoints for individual VPG

    getCheckpointStatsForVpg(vpgid)
        Returns earliest and latest checkpoint statistics for individual VPG

    getValidValuesForVpgEntities()
        Returns all valid values for VPG entity types

    getValidValuesForFailoverCommitPolicies()
        Returns all valid values for Failover commit polices

    getValidValuesForFailoverShutdownPolicies()
        Returns all valid values for Failover shutdown policies

    getValidValuesForVpgPriorities()
        Returns all valid values for VPG priorities

    getValidValuesForFailoverRetentionPolicies()
        Returns all valid values for Failover retention policies
    
    getValidValuesForVpgStatuses()
        Returns all valid values for VPG statuses

    getValidValuesForVpgSubstatuses()
        Returns all valid values for VPG substatuses
    
    generatePeeringToken()
        Generates site pairing token for pairing to ZVM / ZCA
    
    insertTaggedCheckpoint(vpgid, body)
        Inserts tagged checkpoint for individual VPG

    cloneVpg(vpgid, body)
        Performs Clone operation for individual VPG
    
    cloneAbort(vpgid, body)
        Aborts Clone operation for individual VPG

    failoverVpg(vpgid, body)
        Performs Failover operation for individual VPG

    commitFailover(vpgid, body)
        Performs commit of Failover operation for individual VPG
    
    rollbackFailover(vpgid, body)
        Performs rollback of Failover operation for individual VPG
    
    failoverTest(vpgid, body)
        Performs Failover Test operation for individual VPG

    stopFailoverTest(vpgid, body)
        Stops Failover Test operation for individual VPG

    forceSyncVpg(vpgid, body)
        Performs Force Sync operation for individual VPG

    moveVpg(pgid, body)
        Performs Move operation for individual VPG
    
    rollbackMoveVpg(vpgid, body)
        Rollback Move operation for individual VPG
    
    commitMove(vpgid, body)
        Performs a commit of Move operation for individual VPG

    pauseVpgProtection(vpgid, body)
        Pause VPG protection for individual VPG

    resumeVpgProtection(vpgid, body)
        Resume VPG protection for individual VPG

    deleteVpg(vpgid, body)
        Delete individual VPG 
    """
    endPoint = '/vpgs'

    def __init__(self, zvmip, headerwithkey, vpgid=None):
        """
        Parameters
        ----------
        zvmip : str
            The IP of the ZVM or ZCA
        headerwithkey : dict
            A properly formatted dict containing the following key:value pairs:
                {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                    'x-zerto-session': str-type containing valid session key generated with zerto_auth.py
                }
        vpgid: str
            unique identifier for individual VPG
        """

        self.zvmip = 'https://' + zvmip + ':9669/v1'
        self.headerwithkey = headerwithkey
        self.vpgid = vpgid

    def getInfoAllVpgs(self):
        """
        Returns information on all VPGs
        
        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint, headers=self.headerwithkey, verify=False)

    def getInfoSingleVpg(self, vpgid):
        """
        Returns information for individual VPG
        
        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want information returned on 

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid, headers=self.headerwithkey, verify=False)

    def getCheckpointsForVpg(self, vpgid):
        """
        Returns all checkpoints for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on 

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/checkpoints', headers=self.headerwithkey, verify=False)

    def getCheckpointStatsForVpg(self, vpgid):
        """
        Returns earliest and latest checkpoint statistics for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on 

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/checkpoints/stats', headers=self.headerwithkey, verify=False)

    def getValidValuesForVpgEntities(self):
        """
        Returns all valid values for VPG entity types
        
        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/entitytypes', headers=self.headerwithkey, verify=False)
    
    def getValidValuesForFailoverCommitPolicies(self):
        """
        Returns all valid values for Failover commit polices
        
        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/failovercommitpolicies', headers=self.headerwithkey, verify=False)
        
    def getValidValuesForFailoverShutdownPolicies(self):
        """
        Returns all valid values for Failover shutdown polices
        
        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/failovershutdownpolicies', headers=self.headerwithkey, verify=False)
        
    def getValidValuesForVpgPriorities(self):
        """
        Returns all valid values for VPG priorities
        
        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/priorities', headers=self.headerwithkey, verify=False)

    def getValidValuesForFailoverRetentionPolicies(self):
        """
        Returns all valid values for Failover Retention polices
        
        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/retentionpolicies', headers=self.headerwithkey, verify=False)

    def getValidValuesForVpgStatuses(self):
        """
        Returns all valid values for VPG Statuses
        
        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/statuses', headers=self.headerwithkey, verify=False)
        
    def getValidValuesForVpgSubstatuses(self):
        """
        Returns all valid values for VPG Substatuses
        
        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/substatuses', headers=self.headerwithkey, verify=False)
        
    def generatePeeringToken(self):
        """
        Generates site pairing token for pairing to ZVM / ZCA
        
        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmip + self.endPoint + '/generatetoken', headers=self.headerwithkey, verify=False)
        
    def insertTaggedCheckpoint(self, vpgid, body):
        """
        Inserts tagged checkpoint for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on

        body: dict, required
            A properly formatted dict containing the following key:value pairs:
                {  
                    "checkpointName": "String content"
                }
        
        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/Checkpoints', body = body, headers=self.headerwithkey, verify=False)
        
    def cloneVpg(self, vpgid, body):
        """
        Performs Clone operation for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on

        body: dict, required
            A properly formatted dict containing the following key:value pairs:
                {  
                    "CheckpointId": "String content",
                    "DatastoreIdentifier": "String content",
                    "VmIdentifiers":
                    [  
                        VmIdentifier": "string content"
                    ]
                }
        
        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/CloneStart', body = body, headers=self.headerwithkey, verify=False)
        
    def cloneAbort(self, vpgid):
        """
        Aborts Clone operation for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on 

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/CloneAbort', headers=self.headerwithkey, verify=False)

    def failoverVpg(self, vpgid, body):
        """
        Performs Failover operation for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on

        body: dict, required
            A properly formatted dict containing the following key:value pairs:
            {  
                "CheckpointIdentifier": "String content",
                "CommitPolicy":1,
                "ShutdownPolicy":0,
                "TimeToWaitBeforeShutdownInSec":2147483647,
                "IsReverseProtection": Boolean,
                "VmIdentifiers":
                [  
                    VmIdentifier": "string content"
                ]
            }

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/Failover', body = body, headers=self.headerwithkey, verify=False)
       
    def commitFailover(self, vpgid, body):
        """
        Commit Failover operation for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on

        body: dict, required
        A properly formatted dict containing the following key:value pairs:
            {  
            "IsReverseProtection": Boolean
            }

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/FailoverCommit', body = body, headers=self.headerwithkey, verify=False)
        
    def rollbackFailover(self, vpgid):
        """
        Rollback Failover operation for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on

        Returns
        -------
        type requests.models.Response object
        """        
        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/FailoverRollback', headers=self.headerwithkey, verify=False)
       

    def failoverTest(self, vpgid, body):
        """
        Perform Failover Test operation for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on

        body: dict, required
        A properly formatted dict containing the following key:value pairs:
            {  
                "CheckpointIdentifier": "String content",
                "VmIdentifiers":
                    [  
                        VmIdentifier": "string content"
                    ]
            }

        Returns
        -------
        type requests.models.Response object
        """
        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/FailoverTest', body = body, headers=self.headerwithkey, verify=False)

    def stopFailoverTest(self, vpgid, body):
        """
        Stop Failover Test operation for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on

        body: dict, required
        A properly formatted dict containing the following key:value pairs:
            {  
                "FailoverTestSuccess": Boolean,
                "FailoverTestSummary": "String content"
            }

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/FailoverTestStop', body = body, headers=self.headerwithkey, verify=False)
        
    def forceSyncVpg(self, vpgid):
        """
        Perform Force Sync operation for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on

        Returns
        -------
        type requests.models.Response object
        """           
        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/forcesync', headers=self.headerwithkey, verify=False)

    def moveVpg(self, vpgid, body):
        """
        Perform Move operation for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on

        body: dict, required
        A properly formatted dict containing the following key:value pairs:
        {  
            "CommitPolicy": "String content",
            "CommitPolicyTimeout": 2,
            "ForceShutdown": Boolean,
            "ReverseProtection": Boolean,
            "KeepSourceVms": Boolean,
            "ContinueOnPreScriptFailure": Boolean
        }

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/move', body = body, headers=self.headerwithkey, verify=False)

    def rollbackMoveVpg(self, vpgid, body):
        """
        Rollback Failover operation for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on

        Returns
        -------
        type requests.models.Response object
        """       

        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/moveRollback', body = body, headers=self.headerwithkey, verify=False)

    def commitMove(self, vpgid, body):
        """
        Commit Move operation for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on

        body: dict, required
        {  
            "ReverseProtection": Boolean,
            "KeepSourceVms": Boolean
        }

        Returns
        -------
        type requests.models.Response object
        """        
        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/moveCommit', body = body, headers=self.headerwithkey, verify=False)

    def pauseVpgProtection(self, vpgid):
        """
        Pause protection for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/pause', headers=self.headerwithkey, verify=False)

    def resumeVpgProtection(self, vpgid):
        """
        Resume protection for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/resume', headers=self.headerwithkey, verify=False)

    def deleteVpg(self, vpgid, body):
        """
        Delete VPG for individual VPG

        Parameters
        ----------
        vpgid : str, required
            The VPG identifier that you want checkpoint information returned on

        body: dict, required
        {  
            "Force": Boolean,
            "KeepRecoveryVolumes": Boolean
        }

        Returns
        -------
        type requests.models.Response object
        """           
        return requests.delete(self.zvmip + self.endPoint + '/' + vpgid, data=body, headers=self.headerwithkey, verify=False)

class vpgSettings():
    """
    the vpgSettings class houses VPG "object" specific methods. Unlike the vpg class, the methods in vpgSettings apply
    to a VPG as a whole as opposed to what is inside a VPG. Use vpgSettings to create and commit VPGs into existence,
    get settings, and the like.
    ...
    Attributes
    ----------
    zvmip : str
        the IP address of the target ZVM or ZCA

    headerwithkey : dict
        a properly formatted dict containing the following key:value pairs:
        {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
            'x-zerto-session': str-type containing valid session key generated with zerto_auth.py
        }

    vpgid : str (default: None)
        identifier for a VPG

    nicid : str (default: None)
        identifier for a single NIC

    volumeid : str (default: None)
        identifier of a volume

    Methods
    -------
    getSettingsForVpgs()
        Returns settings information for all VPGs

    getSettingsForSingleVpg(vpgid)
        Returns settings information for a single specified VPG

    getBasicVpgSettings(vpgid)
        Returns basic (ie, no granular VM) settings for a single specific VPG.

    getBootVpgSettings(vpgid)
        Returns boot group settings for a single specified VPG

    getJournalVpgSettings(vpgid)
        Returns journal settings for a single specified VPG

    getLtrVpgSettings(vpgid)
        Returns LTR settings information for a single specified VPG

    getNetworkVpgSettings(vpgid)
        Returns network settings for a single specified VPG

    getVpgPriority(vpgid)
        Returns the priority level for a single specified VPG

    getVpgRecoverySettings(vpgid)
        Returns recovery settings for a single specified VPG

    getVpgScriptSettings(vpgid)
        Returns pre- and post-recovery scripting information for a single specified VPG

    getAllVmSettingsForAVpg(vpgid)
        Returns all VM-related settings for a single specified VPG

    getVMSettingsForSingleVmInVpg(vpgid, vmid)
        Returns all settings for a single specified VM in a single specified VPG

    getNICSettingsForSingleVmInVpg(vpgid, vmid)
        Returns NIC settings for a single specified VM in a single specified VPG

    getSingleNICSettingsForSingleVmInVpg(vpgid, vmid, nicid)
        Returns settings for a single specified NIC in a single specified VM in a single specified VPG.

    getVolumeSettingsForSingleVmInVpg(vpgid, vmid)
        Returns volume settings for a single specified VM in a single specified VPG

    getSingleVolumeSettingsForSingleVmInVpg(vpgid, vmid, volumeid)
        Returns information about a single specified volume about a single specified VM in a single specified VPG

    createNewVpgSettingsObject(argbody)
        Creates an empty VPG settings object. This object is the foundation of a new VPG.

    createDefaultVpgLTRSettings(body)
        Creates a default LTR settings VPG object.

    commitSettingsObject(vpgid)
        Commits (closes) and executes the actual creation of a VPG after settings have been applied to the vpgid.

    addVmsToSettingObject(vpgid, body)
        Adds VMs to an open VPG settings object.

    editVpgSettingsObject(vpgid, argbody)
        Edits an open VPG settings object with the contents of argbody.

    editBasicSettingsObject(vpgid, body)
        Edits an open BasicSettingsObject with the contents of argbody.

    editBootSettingsObject(vpgid, body)
        Edits the boot group of an open VPG.

    editJournalObject(vpgid, body)
        Edits the journal of an open VPG.

    editLtrSettings(vpgid, body)
        Edits the LTR settings of an open VPG.

    editNetworksObject(vpgid, body)
        Edits the network settings of an open VPG.

    editRecoveryObject(vpgid, body)
        Edits the recovery objects of an open VPG.

    editScriptingObject(vpgid, body)
        Edits the scripting objects of an open VPG.

    editVmSettingsObject(vpgid, vmid, body)
        Edits the VM settings object of an open VPG.

    editNicSettingsObject(vpgid, vmid, nicid, body)
        Edits the NIC settings object of an open VPG.

    editVolumeSettingsObject(vpgid, vmid, volumeid, body)
        Edits the volume settings object of an open VPG.

    deleteSettingsObject(vpgid)
        Deletes a VPG Settings object.

    deleteBasicSettingsObject(vpgid)
        Deletes a BasicSettings object.

    deleteBootSettingsObject(vpgid)
        Deletes a boot settings object.

    deleteJournalSettingsObject(vpgid)
        Deletes a journal settings object.

    deleteLTRSettingsObject(vpgid)
        Deletes an LTR Settings object.

    deleteNetworkSettingsObject(vpgid)
        Deletes a Network Settings object.

    deleteRecoverySettingsObject(vpgid)
        Deletes a recovery settings object.

    deleteScriptSettingsObject(vpgid)
        Deletes a script settings object.

    deleteVMSettingsFromAVpg(vpgid, vmid)
        Deletes a VMSettings object.

    deleteNicSettingsObject(vpgid, vmid, nicid)
        Deletes a NIC settings object.
    """

    endPoint = '/vpgSettings'

    def __init__(self, zvmip, headerwithkey, vpgid=None, vmid=None, nicid=None, volumeid=None):
        """
        Parameters
        ----------
        zvmip : str, required
            IP address of target ZVM or ZCA

        headerwithkey : dict, required
            A properly formatted dict containing the following key:value pairs:
                {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                    'x-zerto-session': str-type containing valid session key generated with zerto_auth.py
                }

        vpgid : str, optional
            ID of the VPG in question

        vmid : str, optional
            ID of the VM in question

        nicid : str, optional
            ID of the NIC in question

        volumeid : str, optional
            ID of the volume in question
        """
        self.zvmip = 'https://' + zvmip + ':9669/v1'
        self.headerwithkey = headerwithkey
        self.vpgid = vpgid
        self.vmid = vmid
        self.nicid = nicid
        self.volumeid = volumeid


    def getSettingsForVpgs(self):
        """
        Returns settings information for all VPGs

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint, headers=self.headerwithkey, verify=False)

    def getSettingsForSingleVpg(self, vpgid):
        """
        Returns settings information for a single specified VPG

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid, headers=self.headerwithkey, verify=False)

    def getBasicVpgSettings(self, vpgid):
        """
        Returns basic (ie, no granular VM) settings for a single specific VPG.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/basic', headers=self.headerwithkey,
                            verify=False)

    def getBootVpgSettings(self, vpgid):
        """
        Returns boot group settings for a single specified VPG

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/bootgroup', headers=self.headerwithkey,
                            verify=False)

    def getJournalVpgSettings(self, vpgid):
        """
        Returns journal settings for a single specified VPG

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/journal', headers=self.headerwithkey,
                            verify=False)

    def getLtrVpgSettings(self, vpgid):
        """
        Returns LTR settings information for a single specified VPG

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/ltr', headers=self.headerwithkey, verify=False)

    def getNetworkVpgSettings(self, vpgid):
        """
        Returns network settings for a single specified VPG

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/networks', headers=self.headerwithkey, verify=False)

    def getVpgPriority(self, vpgid):
        """
        Returns the priority level for a single specified VPG

        Parameters
        ----------
        vpgid: str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/priority', headers=self.headerwithkey, verify=False)

    def getVpgRecoverySettings(self, vpgid):
        """
        Returns recovery settings for a single specified VPG

        Parameters
        ----------
        vpgid: str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/recovery', headers=self.headerwithkey, verify=False)

    def getVpgScriptSettings(self, vpgid):
        """
        Returns pre- and post-recovery scripting information for a single specified VPG

        Parameters
        ----------
        vpgid: str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/scripting', headers=self.headerwithkey, verify=False)

    def getAllVmSettingsForAVpg(self, vpgid):
        """
        Returns all VM-related settings for a single specified VPG

        Parameters
        ----------
        vpgid: str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/vms', headers=self.headerwithkey, verify=False)

    def getVMSettingsForSingleVmInVpg(self, vpgid, vmid):
        """
        Returns all settings for a single specified VM in a single specified VPG

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        vmid : str, required
            ID of the VM in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/vms/' + vmid, headers=self.headerwithkey, verify=False)

    def getNICSettingsForSingleVmInVpg(self, vpgid, vmid):
        """
        Returns NIC settings for a single specified VM in a single specified VPG

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        vmid : str, required
            ID of the VM in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/vms/' + vmid + '/nics', headers=self.headerwithkey, verify=False)

    def getSingleNICSettingsForSingleVmInVpg(self, vpgid, vmid, nicid):
        """
        Returns settings for a single specified NIC in a single specified VM in a single specified VPG.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        vmid : str, required
            ID of the VM in question
        nicid : str, required
            ID of the NIC in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/vms/' + vmid + '/nics/' + nicid,
                                headers=self.headerwithkey, verify=False)

    def getVolumeSettingsForSingleVmInVpg(self, vpgid, vmid):
        """
        Returns volume settings for a single specified VM in a single specified VPG

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        vmid : str, required
            ID of the VM in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/vms/' + vmid + '/volumes',
                                headers=self.headerwithkey, verify=False)

    def getSingleVolumeSettingsForSingleVmInVpg(self, vpgid, vmid, volumeid):
        """
        Returns information about a single specified volume about a single specified VM in a single specified VPG

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        vmid : str, required
            ID of the VM in question
        volumeid : str, required
            ID of the volume in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmip + self.endPoint + '/' + vpgid + '/vms/' + vmid + '/volumes/' + volumeid,
                                headers=self.headerwithkey, verify=False)

    def createNewVpgSettingsObject(self, argbody):
        """
        Creates an empty VPG settings object. This object is the foundation of a new VPG.

        Parameters
        ----------
        argbody : dict, required
            Properly formatted JSON body. See Zerto API documentation:
            http://s3.amazonaws.com/zertodownload_docs/Latest/Zerto%20Virtual%20Replication%20RESTful%20APIs.pdf
        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmip + self.endPoint, data=argbody, headers=self.headerwithkey, verify=False)

    def createDefaultVpgLTRSettings(self, vpgid, body):
        """
        Creates a default LTR settings VPG object.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        body : dict, required
            Properly formatted JSON body. See Zerto API documentation:
            http://s3.amazonaws.com/zertodownload_docs/Latest/Zerto%20Virtual%20Replication%20RESTful%20APIs.pdf

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/ltr', body=body, headers=self.headerwithkey, verify=False)

    def commitSettingsObject(self, vpgid):
        """
        Commits (closes) and executes the actual creation of a VPG after settings have been applied to the vpgid.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/commit', headers=self.headerwithkey, verify=False)

    def addVmsToSettingObject(self, vpgid, body):
        """
        Adds VMs to an open VPG settings object.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        body : dict, required
            Properly formatted JSON body. See Zerto API documentation:
            http://s3.amazonaws.com/zertodownload_docs/Latest/Zerto%20Virtual%20Replication%20RESTful%20APIs.pdf

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmip + self.endPoint + '/' + vpgid + '/vms', body=body, headers=self.headerwithkey, verify=False)

    def editVpgSettingsObject(self, vpgid, argbody):
        """
        Edits an open VPG settings object with the contents of argbody.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        argbody : dict, required
            Properly formatted JSON body. See Zerto API documentation:
            http://s3.amazonaws.com/zertodownload_docs/Latest/Zerto%20Virtual%20Replication%20RESTful%20APIs.pdf

        Returns
        -------
        type requests.models.Response object
        """

        return requests.put(self.zvmip + self.endPoint + '/' + vpgid, data=argbody, headers=self.headerwithkey, verify=False)

    def editBasicSettingsObject(self, vpgid, body):
        """
        Edits an open BasicSettingsObject with the contents of argbody.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        body : dict, required
            Properly formatted JSON body. See Zerto API documentation:
            http://s3.amazonaws.com/zertodownload_docs/Latest/Zerto%20Virtual%20Replication%20RESTful%20APIs.pdf

        Returns
        -------
        type requests.models.Response object
        """

        return requests.put(self.zvmip + self.endPoint + '/' + vpgid + '/basic', body=body, headers=self.headerwithkey, verify=False)

    def editBootSettingsObject(self, vpgid, body):
        """
        Edits the boot group of an open VPG.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        body : dict, required
            Properly formatted JSON body. See Zerto API documentation:
            http://s3.amazonaws.com/zertodownload_docs/Latest/Zerto%20Virtual%20Replication%20RESTful%20APIs.pdf

        Returns
        -------
        type requests.models.Response object
        """

        return requests.put(self.zvmip + self.endPoint + '/' + vpgid + '/bootgroup', body=body, headers=self.headerwithkey, verify=False)

    def editJournalObject(self, vpgid, body):
        """
        Edits the journal of an open VPG.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        body : dict, required
            Properly formatted JSON body. See Zerto API documentation:
            http://s3.amazonaws.com/zertodownload_docs/Latest/Zerto%20Virtual%20Replication%20RESTful%20APIs.pdf

        Returns
        -------
        type requests.models.Response object
        """

        return requests.put(self.zvmip + self.endPoint + '/' + vpgid + '/journal', body=body, headers=self.headerwithkey, verify=False)

    def editLtrSettings(self, vpgid, body):
        """
        Edits the LTR settings of an open VPG.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        body : dict, required
            Properly formatted JSON body. See Zerto API documentation:
            http://s3.amazonaws.com/zertodownload_docs/Latest/Zerto%20Virtual%20Replication%20RESTful%20APIs.pdf

        Returns
        -------
        type requests.models.Response object
        """

        return requests.put(self.zvmip + self.endPoint + '/' + vpgid + '/ltr', body=body, headers=self.headerwithkey, verify=False)

    def editNetworksObject(self, vpgid, body):
        """
        Edits the network settings of an open VPG.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        body : dict, required
            Properly formatted JSON body. See Zerto API documentation:
            http://s3.amazonaws.com/zertodownload_docs/Latest/Zerto%20Virtual%20Replication%20RESTful%20APIs.pdf

        Returns
        -------
        type requests.models.Response object
        """

        return requests.put(self.zvmip + self.endPoint + '/' + vpgid + '/networks', body=body, headers=self.headerwithkey, verify=False)

    def editRecoveryObject(self, vpgid, body):
        """
        Edits the recovery objects of an open VPG.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        body : dict, required
            Properly formatted JSON body. See Zerto API documentation:
            http://s3.amazonaws.com/zertodownload_docs/Latest/Zerto%20Virtual%20Replication%20RESTful%20APIs.pdf

        Returns
        -------
        type requests.models.Response object
        """

        return requests.put(self.zvmip + self.endPoint + '/' + vpgid + '/recovery', body=body, headers=self.headerwithkey, verify=False)

    def editScriptingObject(self, vpgid, body):
        """
        Edits the scripting objects of an open VPG.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        body : dict, required
            Properly formatted JSON body. See Zerto API documentation:
            http://s3.amazonaws.com/zertodownload_docs/Latest/Zerto%20Virtual%20Replication%20RESTful%20APIs.pdf

        Returns
        -------
        type requests.models.Response object
        """

        return requests.put(self.zvmip + self.endPoint + '/' + vpgid + '/scripting', body=body, headers=self.headerwithkey, verify=False)

    def editVmSettingsObject(self, vpgid, vmid, body):
        """
        Edits the VM settings object of an open VPG.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        vmid : str, required
            ID of the VM in question
        body : dict, required
            Properly formatted JSON body. See Zerto API documentation:
            http://s3.amazonaws.com/zertodownload_docs/Latest/Zerto%20Virtual%20Replication%20RESTful%20APIs.pdf

        Returns
        -------
        type requests.models.Response object
        """

        return requests.put(self.zvmip + self.endPoint + '/' + vpgid + '/vms/' + vmid, body=body, headers=self.headerwithkey, verify=False)

    def editNicSettingsObject(self, vpgid, vmid, nicid, body):
        """
        Edits the NIC settings object of an open VPG.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        vmid : str, required
            ID of the VM in question
        nicid : str, required
            ID of the NIC in question
        body : dict, required
            Properly formatted JSON body. See Zerto API documentation:
            http://s3.amazonaws.com/zertodownload_docs/Latest/Zerto%20Virtual%20Replication%20RESTful%20APIs.pdf

        Returns
        -------
        type requests.models.Response object
        """

        return requests.put(self.zvmip + self.endPoint + '/' + vpgid + '/vms/' + vmid + '/nics' + nicid, body=body, headers=self.headerwithkey, verify=False)

    def editVolumeSettingsObject(self, vpgid, vmid, volumeid, body):
        """
        Edits the volume settings object of an open VPG.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question
        vmid : str, required
            ID of the VM in question
        volumeid : str, required
            ID of the volume in question
        body : dict, required
            Properly formatted JSON body. See Zerto API documentation:
            http://s3.amazonaws.com/zertodownload_docs/Latest/Zerto%20Virtual%20Replication%20RESTful%20APIs.pdf

        Returns
        -------
        type requests.models.Response object
        """

        return requests.put(self.zvmip + self.endPoint + '/' + vpgid + '/vms/' + vmid + '/volumes/' + volumeid, body=body, headers=self.headerwithkey, verify=False)

    def deleteSettingsObject(self, vpgid):
        """
        Deletes a VPG Settings object.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.delete(self.zvmip + self.endPoint + '/' + vpgid, headers=self.headerwithkey, verify=False)

    def deleteBasicSettingsObject(self, vpgid):
        """
        Deletes a BasicSettings object.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.delete(self.zvmip + self.endPoint + '/' + vpgid + '/basic', headers=self.headerwithkey, verify=False)

    def deleteBootSettingsObject(self, vpgid):
        """
        Deletes a boot settings object.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.delete(self.zvmip + self.endPoint + '/' + vpgid + '/bootgroup', headers=self.headerwithkey, verify=False)

    def deleteJournalSettingsObject(self, vpgid):
        """
        Deletes a journal settings object.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.delete(self.zvmip + self.endPoint + '/' + vpgid + '/journal', headers=self.headerwithkey, verify=False)

    def deleteLTRSettingsObject(self, vpgid):
        """
        Deletes an LTR Settings object.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.delete(self.zvmip + self.endPoint + '/' + vpgid + '/ltr', headers=self.headerwithkey, verify=False)

    def deleteNetworkSettingsObject(self, vpgid):
        """
        Deletes a Network Settings object.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.delete(self.zvmip + self.endPoint + '/' + vpgid + '/networks', headers=self.headerwithkey, verify=False)

    def deleteRecoverySettingsObject(self, vpgid):
        """
        Deletes a recovery settings object.

        Parameters
        ----------
        vpgid : str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.delete(self.zvmip + self.endPoint + '/' + vpgid + '/recovery', headers=self.headerwithkey, verify=False)

    def deleteScriptSettingsObject(self, vpgid):
        """
        Deletes a script settings object.

        Parameters
        ----------
        vpgid: str, required
            ID of the VPG in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.delete(self.zvmip + self.endPoint + '/' + vpgid + '/scripting', headers=self.headerwithkey, verify=False)

    def deleteVMSettingsFromAVpg(self, vpgid, vmid):
        """
        Deletes a VMSettings object.

        Parameters
        ----------
        vpgid: str, required
            ID of the VPG in question
        vmid: str, required
            ID of the VM in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.delete(self.zvmip + self.endPoint + '/' + vpgid + '/vms/' + vmid, headers=self.headerwithkey, verify=False)

    def deleteNicSettingsObject(self, vpgid, vmid, nicid):
        """
        Deletes a NIC settings object.

        Parameters
        ----------
        vpgid: str, required
            ID of the VPG in question
        vmid: str, required
            ID of the VM in question
        nicid: str, required
            ID of the NIC in question

        Returns
        -------
        type requests.models.Response object
        """

        return requests.delete(self.zvmip + self.endPoint + '/' + vpgid + '/vms/' + vmid + '/nics/' + nicid, headers=self.headerwithkey, verify=False)



"""
The zvm.py module contains properly-formatted API calls for various Zerto Virtual Manager (or ZCA Zerto Virtual Manager
 service) tasks, including alert management, site pairing, site information gathering, and license management.
"""

import json
import requests


class zvm:
    """
    The zvm class houses ZVM-specific methods.

    ...

    Attributes
    ----------
    zvmip : str
        the IP address of the target ZVM

    headerwithkey : dict
        a properly formatted dict containing the following key:value pairs:
        {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
            'x-zerto-session': str-type containing valid session key generated with zerto_auth.py
        }

    Methods
    -------
    infoAllAlerts(alertid=None)
        Returns information on all alerts

    infoSingleAlert(alertid)
        Returns information on a single alert

    validAlertEntitiesValues()
        Returns information on valid alert entities

    validAlertHelpIdentifiersValues()
        Returns information on valid alert help identifiers

    validAlertLevelValues()
        Returns information on valid alert levels

    dismissAlert(alertid)
        Dismisses an active alert

    undismissAlert(alertid)
        "Undismisses" (makes active) a previously-acknowledged/dismissed alert

    datastoreInfo()
        Returns information on locally available datastores for use as a replication target.

    datastoreSingleInfo(datastoreid)
        Returns information on a specific datastore locally available for use as a replication target.

    allEvents()
        Returns information on all (recent) events stored by the ZVM.

    filteredEvents()
        UNDER CONSTRUCTION. Function not yet usable.

    getSingleEvent(eventid)
        Returns information on a single event currently stored by the ZVM.

    getValidEventCategories()
        Returns all valid event categories.

    getValidEventEntities()
        Returns valid event entities.

    getValidEventTypes()
        Returns valid event types.

    addLicense(licensekey)
        Accepts a str parameter containing a valid Zerto license key and applies it to the target ZVM or ZCA.

    delLicense()
        Removes the license currently applied to the target ZVM or ZCA.

    getLicense()
        Returns information on the license applied to the target ZVM or ZCA.

    getLocalSiteInfo()
        Returns local site information.

    getValidPairingStatus()
        Returns information on the pairing status of the target ZVM or ZCA to other sites (ie other ZVMs or ZCAs).

    sendBillingData()
        Sends billing data to Zerto Analytics. Meant for use by Zerto Cloud Provider partners.

    getListOfPeerSites()
        Returns information about all peer sites of the target ZVM/ZCA.

    getSinglePeerSites(siteidentifier)
        Returns information about a specific peer site of the target ZVM/ZCA.

    getStatusOfPeerSites()
        Returns information on the status of all peer sites of the target ZVM/ZCA.

    generatePeeringToken()
        Returns a peering token that can then be passed to a desired peer site for the sake of the peering handshake.

    pairToSite()
        UNDER CONSTRUCTION. Function not yet usable.

    getRecoveryReport()
        Returns a recovery report for the target ZVM/ZCA.

    getResourcesReport()
        Returns a resources report for the target ZVM/ZCA.

    getAllTasks()
        Returns all tasks for the recovery ZVM/ZCA.

    getSingleTask(taskidentifier)
        Returns information on a single specified task for the ZVM/ZCA.

    getValidTaskTypes()
        Returns information on valid task types.

    getAllVirtualizationSites()
        Returns information on all virtualization sites.

    getSingleVirtualizationSite(siteidentifier)
        Returns information on a single specified virtualization site.

    getStorageClustersAtSite(siteidentifier)
        Returns information about storage clusters at a specific site available to the ZVM.

    getStorageAtSite(siteidentifier)
        Returns information about all storage at the specified site available to the ZVM/ZCA.

    getDevicesAtAllHosts(siteidentifier)
        Returns information about all devices at all hosts at the specified site to the ZVM/ZCA.

    getDevicesAtSingleHost(siteidentifier, hostidentifier)
        Returns information about all devices at a single host at the specified site to the ZVM/ZCA.

    getFoldersAtSite(siteidentifier)
        Returns information about all folders at the specified site.

    getHostClustersAtSite(siteidentifier)
        Returns information about all host clusters at the specified site.

    getHostsAtSite(siteidentifier)
        Returns information about all hosts at the specified site.

    getSingleHostAtSite(siteidentifier, hostidentifier)
        Returns information about a single host at the specified site.

    getNetworksAtSite(siteidentifier)
        Returns information about all networks at the specified site.

    getRepositoriesAtSite(siteidentifier)
        Returns information about all LTR repositories at the specified site.

    getResourcePoolsAtSite(siteidentifier)
        Returns information about all resource pools at the specified site.

    getUnprotectedVmsAtSite(siteidentifier)
        Returns information about unprotected VMs at the specified site.

    getVmInstanceTypes(siteidentifier)
        Returns information about instance types at the specified site.

    getAvailablevNets(siteidentifier)
        Returns information about available Azure vNets at the specified site.

    getAvailableSubnets(siteidentifier)
        Returns information about available Azure subnets at the specified site.

    getSecurityGroups(siteidentifier)
        Returns information about available security groups at the specified site.

    getAllVolumes()
        Returns information about available volumes visible to the ZVM/ZCA.
    """

    def __init__(self, zvmip, headerwithkey):
        """
        Parameters
        ----------
        zvmip : str
            The IP of the ZVM or ZCA
        headerwithkey : dict
            A properly formatted dict containing the following key:value pairs:
                {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                    'x-zerto-session': str-type containing valid session key generated with zerto_auth.py
                }
        """

        self.zvmurl = 'https://' + zvmip + ':9669/v1'
        self.headerwithkey = headerwithkey

    def infoAllAlerts(self):
        """
        Returns information on all alerts

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/alerts', headers=self.headerwithkey, verify=False)

    def infoSingleAlert(self, alertid):
        """
        Returns information on a single alert

        Parameters
        ----------
        alertid : str, required
            The alert identifier that you want information returned on

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/alerts/' + str(alertid), headers=self.headerwithkey, verify=False)

    def validAlertEntitiesValues(self):
        """
        Returns information on valid alert entities

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/alerts/entities', headers=self.headerwithkey, verify=False)

    def validAlertHelpIdentifiersValues(self):
        """
        Returns information on valid alert help identifiers

        Returns
        -------
        type requests.models.Response object

        """

        return requests.get(self.zvmurl + '/alerts/helpidentifiers', headers=self.headerwithkey, verify=False)

    def validAlertLevelValues(self):
        """
        Returns information on valid alert levels

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/alerts/levels', headers=self.headerwithkey, verify=False)

    def dismissAlert(self, alertid):
        """
        Dismisses an active alert

        Parameters
        ----------
        alertid : str, required
            The specific alert that you want to dismiss

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmurl + '/alerts/levels' + str(alertid) + '/dismiss', headers=self.headerwithkey,
                             verify=False)

    def undismissAlert(self, alertid):
        """
        "Undismisses" (makes active) a previously-acknowledged/dismissed alert

        Parameters
        ----------
        alertid : str, required
            The specific alert that you want to undismiss

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmurl + '/alerts/levels' + str(alertid) + '/undismiss', headers=self.headerwithkey,
                             verify=False)

    def datastoreInfo(self):
        """
        Returns information on locally available datastores for use as a replication target.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/datastores', headers=self.headerwithkey, verify=False)

    def datastoreSingleInfo(self, datastoreid):
        """
        Returns information on a specific datastore locally available for use as a replication target.

        Parameters
        ----------
        datastoreid : str, required
            The specific datastore that you want information on.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/datastores/' + str(datastoreid), headers=self.headerwithkey, verify=False)

    def allEvents(self):
        """
        Returns information on all (recent) events stored by the ZVM.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/events', headers=self.headerwithkey, verify=False)

    # def filteredEvents(self):

        # TODO: Create filteredEvents function

    def getSingleEvent(self, eventid):
        """
        Returns information on a single event currently stored by the ZVM.

        Parameters
        ----------
        eventid : str, required

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/events/' + str(eventid), headers=self.headerwithkey, verify=False)

    def getValidEventCategories(self):
        """
        Returns all valid event categories.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/events/categories', headers=self.headerwithkey, verify=False)

    def getValidEventEntities(self):
        """
        Returns valid event entities.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/events/entities', headers=self.headerwithkey, verify=False)

    def getValidEventTypes(self):
        """
        Returns valid event types.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/events/types', headers=self.headerwithkey, verify=False)

    def addLicense(self, licensekey):
        """
        Accepts a str parameter containing a valid Zerto license key and applies it to the target ZVM or ZCA.

        Parameters
        ----------
        licensekey : str
            The license key you want to apply to the ZVM/ZCA.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.put(self.zvmurl + '/license', data=json.dumps({"LicenseKey": licensekey}),
                            headers=self.headerwithkey, verify=False)

    def delLicense(self):
        """
        Removes the license currently applied to the target ZVM or ZCA.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.delete(self.zvmurl + '/license', headers=self.headerwithkey, verify=False)

    def getLicense(self):
        """
        Returns information on the license applied to the target ZVM or ZCA.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/license', headers=self.headerwithkey, verify=False)

    def getLocalSiteInfo(self):
        """
        Returns local site information.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/localsite', headers=self.headerwithkey, verify=False)

    def getValidPairingStatus(self):
        """
        Returns information on the pairing status of the target ZVM or ZCA to other sites (ie other ZVMs or ZCAs).

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/localsite/pairingstatuses', headers=self.headerwithkey, verify=False)

    def sendBillingData(self):
        """
        Sends billing data to Zerto Analytics. Meant for use by Zerto Cloud Provider partners.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmurl + '/localsite/billing/sendUsage', headers=self.headerwithkey, verify=False)

    def getListOfPeerSites(self):
        """
        Returns information about all peer sites of the target ZVM/ZCA.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/peersites', headers=self.headerwithkey, verify=False)

    def getSinglePeerSites(self, siteidentifier):
        """
        Returns information about a specific peer site of the target ZVM/ZCA.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID that you are wanting to get information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/peersites/' + siteidentifier, headers=self.headerwithkey, verify=False)

    def getStatusOfPeerSites(self):
        """
        Returns information on the status of all peer sites of the target ZVM/ZCA.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/peersites/pairingstatuses', headers=self.headerwithkey, verify=False)

    def generatePeeringToken(self):
        """
        Returns a peering token that can then be passed to a desired peer site for the sake of the peering handshake.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmurl + '/peersites/generatetoken', headers=self.headerwithkey, verify=False)

    def pairToSite(self, peerip, pairingtoken):
        """
        Executes a pairing operation between the target ZVM/ZCA and a requested peer site.

        Parameters
        ----------
        peerip : str, required
            The IP address of the peer site you want to connect to.
        pairingtoken: str, required
            A valid pairing token generated by the desired peer site

        Returns
        -------
        type requests.models.Response object
        """

        return requests.post(self.zvmurl + '/peersites', headers=self.headerwithkey, data=json.dumps(
            {"HostName": peerip, "Port": "9071", "Token": pairingtoken}), verify=False)

    def getRecoveryReport(self):
        """
        Returns a recovery report for the target ZVM/ZCA.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/reports/recovery', headers=self.headerwithkey, verify=False)

    def getResourcesReport(self):
        """
        Returns a resources report for the target ZVM/ZCA.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/reports/resources', headers=self.headerwithkey, verify=False)

    def getAllTasks(self):
        """
        Returns all tasks for the recovery ZVM/ZCA.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/tasks', headers=self.headerwithkey, verify=False)

    def getSingleTask(self, taskidentifier):
        """
        Returns information on a single specified task for the ZVM/ZCA.

        Parameters
        ----------
        taskidentifier : str, required
            The task ID that you want information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/tasks/' + taskidentifier, headers=self.headerwithkey, verify=False)

    def getValidTaskTypes(self):
        """
        Returns information on valid task types.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/tasks/types', headers=self.headerwithkey, verify=False)

    def getAllVirtualizationSites(self):
        """
        Returns information on all virtualization sites.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites', headers=self.headerwithkey, verify=False)

    def getSingleVirtualizationSite(self, siteidentifier):
        """
        Returns information on a single specified virtualization site.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want info about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier, headers=self.headerwithkey,
                            verify=False)

    def getStorageClustersAtSite(self, siteidentifier):
        """
        Returns information about storage clusters at a specific site available to the ZVM.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want storage cluster information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/datastoreclusters',
                            headers=self.headerwithkey, verify=False)

    def getStorageAtSite(self, siteidentifier):
        """
        Returns information about all storage at the specified site available to the ZVM/ZCA.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want storage information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/datastores',
                            headers=self.headerwithkey, verify=False)

    def getDevicesAtAllHosts(self, siteidentifier):
        """
        Returns information about all devices at all hosts at the specified site to the ZVM/ZCA.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want device information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/devices',
                            headers=self.headerwithkey, verify=False)

    def getDevicesAtSingleHost(self, siteidentifier, hostidentifier):
        """
        Returns information about all devices at a single host at the specified site to the ZVM/ZCA.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want device information about.

        hostidentifier : str, required
            The host ID of the specific host you want device information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/devices?hostIdentifier=' +
                            hostidentifier, headers=self.headerwithkey, verify=False)

        # TODO: Verify this actually works, because the documentation contains what looks like a typo...
        #  -- Update -- It works. Doc bug...

    def getFoldersAtSite(self, siteidentifier):
        """
        Returns information about all folders at the specified site.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want folder information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/folders',
                            headers=self.headerwithkey, verify=False)

    def getHostClustersAtSite(self, siteidentifier):
        """
        Returns information about all host clusters at the specified site.

        Parameters
        ----------
        siteidentifier : str, required
           The site ID of the site you want host cluster info about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/hostclusters',
                            headers=self.headerwithkey, verify=False)

    def getHostsAtSite(self, siteidentifier):
        """
        Returns information about all hosts at the specified site.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want host information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/hosts',
                            headers=self.headerwithkey, verify=False)

    def getSingleHostAtSite(self, siteidentifier, hostidentifier):
        """
        Returns information about a single host at the specified site.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want host information about.

        hostidentifier : str, required
            The host ID that you want information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/hosts/' + hostidentifier,
                            headers=self.headerwithkey, verify=False)

    def getNetworksAtSite(self, siteidentifier):
        """
        Returns information about all networks at the specified site.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want network information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/networks',
                            headers=self.headerwithkey, verify=False)

    def getRepositoriesAtSite(self, siteidentifier):
        """
        Returns information about all LTR repositories at the specified site.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want LTR repo information about

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/repositories',
                            headers=self.headerwithkey, verify=False)

    def getResourcePoolsAtSite(self, siteidentifier):
        """
        Returns information about all resource pools at the specified site.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want resource pool information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/resourcepools',
                            headers=self.headerwithkey, verify=False)

    def getUnprotectedVmsAtSite(self, siteidentifier):
        """
        Returns information about unprotected VMs at the specified site.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want unprotected VM information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/vms', headers=self.headerwithkey,
                            verify=False)

    def getVmInstanceTypes(self, siteidentifier):
        """
        Returns information about instance types at the specified site.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want VM instance type information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/publicCloud/vmInstanceTypes',
                            headers=self.headerwithkey, verify=False)

    def getAvailablevNets(self, siteidentifier):
        """
        Returns information about available Azure vNets at the specified site.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want vNet information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/publicCloud/virtualNetworks',
                            headers=self.headerwithkey, verify=False)

    def getAvailableSubnets(self, siteidentifier):
        """
        Returns information about available Azure subnets at the specified site.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want subnet information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/publicCloud/subnets',
                            headers=self.headerwithkey, verify=False)

    def getSecurityGroups(self, siteidentifier):
        """
        Returns information about available security groups at the specified site.

        Parameters
        ----------
        siteidentifier : str, required
            The site ID of the site you want security group information about.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/virtualizationsites/' + siteidentifier + '/publicCloud/securityGroups',
                            headers=self.headerwithkey, verify=False)

    def getAllVolumes(self):
        """
        Returns information about available volumes visible to the ZVM/ZCA.

        Returns
        -------
        type requests.models.Response object
        """

        return requests.get(self.zvmurl + '/volumes', headers=self.headerwithkey, verify=False)



"""
The vra.py module contains properly-formatted API calls for VRA related tasks. The
tasks can include installing a vra, uninstalling a vra, editing a VRA, or upgrading
a vra
"""

import json 
import requests
#from zerto_auth import testUrl, testHeaders
class vra():
    """
    The VRA class houses VRA specific methods
    ...
    Attributes
    ----------
    zvmurl : str
        the IP address of the target ZVM

    headerwithkey : dict
        a properly formatted dict containing the following key:value pairs:
        {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
            'x-zerto-session': str-type containing valid session key generated with zerto_auth.py
        }
    
    vra_id: str 
        unique identifier for individual VRA 

    vra_dict: dict
        dictionary object for VRA inputs 

    Methods
    -------
    infoAllVRAs()
        Get info on all VRAs.
    upgradeGroupVRAs()
        Upgrade a group of VRAs which are specified in the vraid body
    upgradeVRA()
        Upgrade individual VRA
    installVRA()
        Install individual VRA
    editVRA()
        Edit individual VRA
    delVRA()
        Uninstall individual VRA
    """
    endPoint = '/vras'
    
    def __init__(self, zvmip, headerwithkey):
        """
        Parameters
        ----------
        zvmip : str
            The IP of the ZVM or ZCA
        headerwithkey : dict
            A properly formatted dict containing the following key:value pairs:
                {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                    'x-zerto-session': str-type containing valid session key generated with zerto_auth.py
                }
        """

        self.zvmurl = 'https://' + zvmip + ':9669/v1'
        self.headerwithkey = headerwithkey 
       
    def infoAllVRAs(self):
        """
        Returns information on all VRAs
        
        Returns
        -------
        type requests.models.Response object
        """        
        return requests.get(self.zvmurl + self.endPoint, headers=self.headerwithkey, verify=False)
 
    def upgradeGroupVRAs(self, vra_id):
        """
                Upgrade a group of VRAs which are specified in the vra_id body

        Parameters
        ----------
        vra_id: dict , required
            A properly formatted dict containing the following key:value pairs:
            {  
                "VraIdentifiers":
                [  
                    "String content",
                    "String content",
                    ...
                    "String content"
                ]
            }

        Returns
        -------
        type requests.models.Response object
        """    
        return requests.post(self.zvmurl + self.endPoint + "/upgrade", headers=self.headerwithkey, data=vra_id, verify=False)
  
    def upgradeVRA(self, vra_id):
        """
        Upgrade individual VRA
        
        Parameters
        ----------
        vra_id: str, required
            the unique identifier for the vra requiring upgrade

        Returns
        -------
        type requests.models.Response object
        """        
        return requests.post(self.zvmurl + self.endPoint +"/" + vra_id + "/upgrade", headers=self.headerwithkey, verify=False)

    def installVRA(self, vra_dict):
        """
        Performs installation of a VRA

        Parameters
        ----------
        vra_dict: dict , required
            A properly formatted dict containing the following key:value pairs:
            {  
                "DatastoreIdentifier": "String content",
                "GroupName": "String content",
                "HostIdentifier": "String content",
                "HostRootPassword": "String content",
                "MemoryInGb":2,
                "NumOfCpus":1,
                "NetworkIdentifier": "String content",
                "UsePublicKeyInsteadOfCredentials": Boolean,
                "PopulatePostInstallation": Boolean,
                "VraNetworkDataApi": {  
                    "DefaultGateway": "String content",
                    "SubnetMask": "String content",
                    "VraIPAddress": "String content",
                    "VraIPConfigurationTypeApi": "String content"
                }
            }

        Returns
        -------
        type requests.models.Response object
        """ 
        return requests.post(self.zvmurl + self.endPoint, headers=self.headerwithkey, data=vra_dict, verify=False)

    def editVRA(self, vra_dict, vra_id):
        """
        Edit exiting VRA

        Parameters
        ----------
        vra_dict: dict , required
        {  
            "GroupName": "String content",
            "HostRootPassword": "String content",
            "UsePublicKeyInsteadOfCredentials": Boolean,
            "VraNetworkDataApi": {  
                "DefaultGateway": "String content",
                "SubnetMask": "String content",
                "VraIPAddress": "String content",
                "VraIPConfigurationTypeApi": "String content"
            }
        }

        vra_id: str, required
            the unique identifier for the vra requiring edit

        Returns
        -------
        type requests.models.Response object
        """         

        return requests.put(self.zvmurl + self.endPoint +"/" + vra_id, headers=self.headerwithkey, data=vra_dict, verify=False)
    
    def delVRA(self, vra_id):
        """
        Uninstall individual VRA
        
        Parameters
        ----------
        vra_id: str, required
            the unique identifier for the vra requiring uninstall

        Returns
        -------
        type requests.models.Response object
        """   
        return requests.delete(self.zvmurl + self.endPoint +"/" + vra_id, headers=self.headerwithkey, verify=False)






"""
The zerto_auth.py module is the main authenticator script that must be run before any other API calls are accepted
by the target Zerto Virtual Manager or Zerto Cloud Appliance.
"""

import requests
from requests.auth import HTTPBasicAuth

def login(zvm_ip, zvm_user, zvm_password):
    """The login function returns a valid header, including an API token, that can then be passed to newly instansiated
    classes from other modules.

    The login function accepts three str-type inputs (IP addresss of ZVM/ZCA, username of ZVM/ZCA, and password of ZVM/
    ZCA) and returns a dict-type containing the valid headers for future API requests.
    """
    sessionUrl = 'https://' + zvm_ip + ':9669/v1/session/add'
    print("Getting API token for " + zvm_ip + "...")
    auth_info = "{\r\n\t\"AuthenticationMethod\":1\r\n}"
    headers = {
        'Accept': 'application/json',
        'Content-Type': 'application/json'
        }

    response = requests.post(sessionUrl, headers=headers, data=auth_info, verify=False, auth=HTTPBasicAuth(zvm_user,
                                                                                                          zvm_password))
    if response.ok:
        auth_token = response.headers['x-zerto-session']
        print("Api Token: " + auth_token)
        headers['x-zerto-session'] = auth_token
        print(headers)
        return headers

    else:
        print("HTTP %i - %s, Message %s" % (response.status_code, response.reason, response.text))


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="zertoapl",
    version="0.0.2",
    author="Alex Schenck and Shaun Finn",
    author_email="linuxbeak@gmail.com, shaun_ryan_finn@yahoo.com",
    description="Zerto Automation API Library",
    long_description="zertoapl is a simple python3 API wrapper for the Zerto product by the eponymous "
                     "corporation. It is intended to simplify the deployment and management of Zerto in a code-driven "
                     "manner. Potential uses for this API wrapper are as diverse as you may wish to make it. An example "
                     "script of automatically protecting tagged virtual machines (VMs) is included in this library.",
    long_description_content_type="text/markdown",
    url="https://github.com/pilotschenck/pyzerto-unofficial",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)